<?php  /* -*- Mode: PHP; tab-width: 4; c-basic-offset: 2 -*- */
#BEGIN_LICENSE
#-------------------------------------------------------------------------
# Module: CompanyDirectory (c) 2004-6 by Ted Kulp (ted@cmsmadesimple.org)
#  
#  Copyright 2006,2008,2009,2010 by Robert Campbell (calguy1000@cmsmadesimple.org)
#
#  An addon module for CMS Made Simple to allow management of and various ways to display
#  company information for use in directories etc.
# 
#-------------------------------------------------------------------------
# CMS - CMS Made Simple is (c) 2005-2010 by Ted Kulp (wishy@cmsmadesimple.org)
# This project's homepage is: http://www.cmsmadesimple.org
#
#-------------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# However, as a special exception to the GPL, this software is distributed
# as an addon module to CMS Made Simple.  You may not use this software
# in any Non GPL version of CMS Made simple, or in any version of CMS
# Made simple that does not indicate clearly and obviously in its admin 
# section that the site was built with CMS Made simple.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# Or read it online: http://www.gnu.org/licenses/licenses.html#GPL
#
#-------------------------------------------------------------------------
#END_LICENSE
if (!isset($gCms)) exit;
if (!$this->CheckPermission('Modify Company Directory'))
{
	echo $this->ShowErrors($this->Lang('needpermission', array('Modify Company Directory')));
	return;
}

if (isset($params['cancel']))
{
  $params = array('tab_message'=> 'categoryupdated', 'active_tab' => 'categories');
  $this->Redirect($id, 'admin_settings', $returnid,$params);
}

$config = cmsms()->GetConfig();
$userid = get_userid();
$tmpdir = '';
$image = null;
$description = '';
$delimage = 0;
$catid = '';
$name = '';
$extra1 = '';
$extra2 = '';
$extra3 = '';

if (!isset($params['catid']))
{
  echo $this->ShowErrors($this->Lang('error_missingparam'));
  return;
}

$catid = (int)$params['catid'];
$catdir = cms_join_path($config['uploads_path'],'companydirectory','cat'.$catid);

// load the info.
$query = 'SELECT * FROM '.cms_db_prefix().'module_compdir_categories WHERE id = ?';
$row = $db->GetRow($query, array($catid));
if ($row)
  {
	$name = $row['name'];
	$image = $row['image'];
	$description = $row['description'];
	$extra1 = $row['extra1'];
	$extra2 = $row['extra2'];
	$extra3 = $row['extra3'];
  }

if (isset($params['submit']))
{
  $error = '';

  // get the info
  if (isset($params['name']))
	{
	  $name = trim($params['name']);
	}

  if( isset($params['description']) )
	{
	  $description = $params['description'];
	}

  if( isset($params['delimage']) )
	{
	  $delimage = 1;
	}

  if (isset($params['extra1']))
	{
	  $extra1 = $params['extra1'];
	}

  if (isset($params['extra2']))
	{
	  $extra2 = $params['extra2'];
	}

  if (isset($params['extra3']))
	{
	  $extra3 = $params['extra3'];
	}

  // validate the info
  if ($name == '')
	{
	  $error = $this->Lang('nonamegiven');
	}
  if( $error == '' && preg_match('/[^\w ]/',$name) )
	{
	  $error = $this->Lang('badnamegiven');
	}


  // handle the info
  if( !$error && $delimage == 1)
	{
	  // delete existing image (and dir)
	  // this would have to change if more than one image was allowed.
	  cge_dir::recursive_rmdir($catdir);
	  $image = null;
	}

  if( !$error )
	{
	  // handle file upload (if any) (to a temporary directory)
	  $tmpdir = cms_join_path($config['uploads_path'],'companydirectory','u'.$userid);
	  $uploader = cge_setup::get_uploader($id,$tmpdir);
	  $res = $uploader->check_upload('image',FALSE,FALSE);
	  if( !$res && $uploader->get_error() != cg_fileupload::NOFILE )
		{
		  $error = $this->GetUploadErrorMessage($uploader->get_error());
		}
	  else
		{
		  cge_dir::mkdirr($tmpdir);
		  $image = $uploader->handle_upload('image');
		  if( !$image && $uploader->get_error() != cg_fileupload::NOFILE )
			{
			  $error = $this->GetUploadErrorMessage($uploader->get_error());
			}
		}
	}

  if( !$error )
	{
	  // database work.
	  $query = 'UPDATE '.cms_db_prefix().'module_compdir_categories 
                SET name = ?, description = ?, image = ?, extra1 = ?, extra2 = ?, extra3 = ?, modified_date = NOW()
                WHERE id = ?';
	  $dbr = $db->Execute($query, array($name, $description, $image, $extra1, $extra2, $extra3, $catid));

	  if( !$dbr )
		{
		  $error = $db->ErrorMsg().' -- '.$db->sql;
		}
	}

  if( !$error && $image && $tmpdir && is_dir($tmpdir) )
	{
	  cge_dir::recursive_rmdir($catdir); // nuke all files... just to be safe, we only allow one atm.
	  cge_dir::mkdirr($catdir);

	  $files = glob($tmpdir.'/*.*');
	  foreach( $files as $file )
		{
		  $bn = basename($file);
		  @rename($file,cms_join_path($catdir,$bn));
		}
	  cge_dir::recursive_rmdir($tmpdir);
	  $tmpdir = '';
	}

  if( !$error )
	{
	  // no errors, get outa here.
	  audit($catid,$this->GetName(),'Edited Category '.$name);
	  $params = array('tab_message'=> 'categoryupdated', 'active_tab' => 'categories');
	  $this->Redirect($id, 'admin_settings', $returnid, $params);			
	}

  // display errors
  if( $tmpdir && is_dir($tmpdir) )
	{
	  cge_dir::recursive_rmdir($tmpdir);
	}
  echo $this->ShowErrors($error);

}

// Display template
$smarty->assign('startform', $this->CreateFormStart($id, 'editcategory', $returnid,'post','multipart/form-data'));
$smarty->assign('endform', $this->CreateFormEnd());
$smarty->assign('image',$image);
$smarty->assign('description',$description);
$smarty->assign('nametext', $this->Lang('name'));
$smarty->assign('inputname', $this->CreateInputText($id, 'name', $name, 20, 255));
$smarty->assign('inputextra1', $this->CreateInputText($id, 'extra1', $extra1, 60, 255));
$smarty->assign('inputextra2', $this->CreateInputText($id, 'extra2', $extra2, 60, 255));
$smarty->assign('inputextra3', $this->CreateInputText($id, 'extra3', $extra3, 60, 255));
$smarty->assign('hidden', $this->CreateInputHidden($id, 'catid', $catid));
$smarty->assign('submit', $this->CreateInputSubmit($id, 'submit', lang('submit')));
$smarty->assign('cancel', $this->CreateInputSubmit($id, 'cancel', lang('cancel')));

echo $this->ProcessTemplate('editcategory.tpl');
?>
