<?php  /* -*- Mode: PHP; tab-width: 4; c-basic-offset: 2 -*- */
#BEGIN_LICENSE
#-------------------------------------------------------------------------
# Module: CompanyDirectory (c) 2004-6 by Ted Kulp (ted@cmsmadesimple.org)
#  
#  Copyright 2006,2008,2009,2010 by Robert Campbell (calguy1000@cmsmadesimple.org)
#
#  An addon module for CMS Made Simple to allow management of and various ways to display
#  company information for use in directories etc.
# 
#-------------------------------------------------------------------------
# CMS - CMS Made Simple is (c) 2005-2010 by Ted Kulp (wishy@cmsmadesimple.org)
# This project's homepage is: http://www.cmsmadesimple.org
#
#-------------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# However, as a special exception to the GPL, this software is distributed
# as an addon module to CMS Made Simple.  You may not use this software
# in any Non GPL version of CMS Made simple, or in any version of CMS
# Made simple that does not indicate clearly and obviously in its admin 
# section that the site was built with CMS Made simple.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# Or read it online: http://www.gnu.org/licenses/licenses.html#GPL
#
#-------------------------------------------------------------------------
#END_LICENSE
if (!isset($gCms)) exit;

$oldsearch = $this->GetPreference('use_oldsearch',0);
if( isset($params['oldsearch']) ) $oldsearch = (int)$params['oldsearch'];
if( $oldsearch ) {
  // using the old search stuff...
  include_once(dirname(__FILE__).'/action.oldsearch.php');
  return;
}


#
# Initialization
#
$resultpage = $returnid;
$errors = array();

#
# Get parameters
#
if( isset($params['resultpage']) ) {
  $tmp = $this->resolve_alias_or_id($params['resultpage']);
  if( $tmp ) $resultpage = $tmp;
  unset($params['resultpage']);
}

#
# Handle form submission
#

if( isset($params['cd_submit']) || isset($params['dosearch']) ) {
  try {
	//
	// first parse of parameters
	//
	$newparms = array();
	$postalparms = array();
	$fieldparms = array();

	foreach( $params as $key => $value ) {
	  if( startswith($key,'cd_') ) $key = substr($key,3);
	  if( !is_array($value) ) $value = trim($value);
	  if( empty($value) ) continue;

	  switch( $key ) {
	  case 'name':
		$newparms[$key] = $value;
		break;

	  case 'address':
	  case 'category':
		$newparms[$key] = $value;
		break;

	  case 'lat':
	  case 'latitude':
		// validate float
		if( $value != '' && (float)$value == 0.0 ) throw new CompanyDirectoryException($this->Lang('error_invalid_param',$key));
		$key = 'lat';
		$newparms[$key] = $value;
		break;

	  case 'long':
	  case 'longitude':
		// validate float
		if( $value != '' && (float)$value == 0.0 ) throw new CompanyDirectoryException($this->Lang('error_invalid_param',$key));
		$key = 'long';
		$newparms[$key] = $value;
		break;

	  case 'name_type':
		$value = strtolower($value);
		if( !in_array($value,array('exact','like')) ) {
		  throw new CompanyDirectoryException($this->Lang('error_invalid_param',$key).': '.$value);
		}
		$newparms[$key] = $value;
		break;

	  case 'addr_type':
	  case 'address_type':
		$value = strtolower($value);
		if( !in_array($value,array('substr','lookup')) ) throw new CompanyDirectoryException($this->Lang('error_invalid_param',$key));
		$key = 'addr_type';
		$newparms[$key] = $value;
		break;

	  case 'radius':
		// validate float.
		if( (float)$value == 0.0 ) throw new CompanyDirectoryException($this->Lang('error_invalid_param',$key).': '.$value);
		$newparms[$key] = $value;
		break;

	  case 'units':
		// validate miles or kilometers
		$value = strtolower($value);
		if( !in_array($value,array('mi','mile','miles','km','k')) ) throw new CompanyDirectoryException($this->Lang('err_invalidunits'));
		$newparms[$key] = $value;
		break;

	  case 'origpage':
	  case 'submit':
	  case 'cancel':
	  case 'resultpage':
	  case 'summarytemplate':
	  case 'detailpage':
	  case 'detailtemplate':
	  case 'action':
	  case 'module':
	  case 'dosearch':
	  case 'cdx_field':
		// do nothing.
		break;

	  case 'postal':
	  case 'postalchars':
	  case 'country':
	  case 'searchaddress':
		// doing a postal search (unless lat/long are also specified)
		$postalparms[$key] = $value;
		break;

	  default:
		$newparms[$key] = $value;
		break;
	  }
	}

	if( isset($params['cdx_field']) && is_array($params['cdx_field']) && count($params['cdx_field']) ) {
	  $value = $params['cdx_field'];
	  $tmp = array();
	  foreach( $value as $one ) {
		if( !is_array($one) ) continue;
		if( !isset($one['fldname']) || empty($one['fldname']) ) continue;
		if( !isset($one['fldval']) || empty($one['fldval']) ) continue;
		if( !isset($one['expr']) || empty($one['expr']) ) $one['expr'] = 'AUTO';
		$tmp[] = $one;
	  }
	  $newparms['field'] = $tmp;
	}

	$can_search = TRUE;

	// do an address lookup if we need to.
	// and we don't alredy have latitude/longitude
	if( isset($newparms['address']) && isset($newparms['addr_type']) && $newparms['addr_type'] == 'lookup' ) {
	  if( !isset($newparms['lat']) && !isset($newparms['long']) ) {
		$coords = cd_utils::geolocate($newparms['address']);
		if( $coords !== false ) {
		  $newparms['lat'] = $coords['lat'];
		  $newparms['long'] = $coords['lon'];
		  unset($newparms['address']); // don't wanna use the address again for filtering.
		}
		else {
		  throw new CompanyDirectoryException($this->Lang('error_address_lookup',$address));
		}
	  }
	}

	// do postcode search if necessary (if we don't alredy have latitude and longitude)
	if( count($postalparms) && isset($postalparms['postal']) && $postalparms['postal'] != '' ) {
	  $searchaddress = 0;
	  if( isset($postalparms['searchaddress']) ) $searchaddress = (int)$postalparms['searchaddress'];
	  if( !isset($newparms['lat']) && !isset($newparms['long']) ) {
		// we don't already have a latitude/longitude
		$postal = $postalparms['postal'];
		$mod = cms_utils::get_module('Postcode');
		if( $mod ) {
		  // found the postcode module.
		  $country = 'US';
		  $postchars = '';
		  if( isset($postalparms['country']) && strlen($postalparms['country']) == 2 ) $country = strtoupper($postalparms['country']);
		  if( isset($postalparms['postchars']) && is_int($postalparms['postchars']) ) $postchars = (int)$postalparms['postchars'];

		  $postal_data = $mod->Lookup_Zip($country,$postal,$postchars);
		  if( !is_array($postal_data) ) {
			// it didn't work
			throw new CompanyDirectoryException($this->Lang('error_postcode_lookup',$postal));
		  }
		  else {
			$newparms['lat'] = $postal_data['latitude'];
			$newparms['long'] = $postal_data['longitude'];
		  }
		}
	  }
	  if( $searchaddress ) $newparms['address2'] = $postal;
	}

	// validate newparms
	if( isset($newparms['name']) ) {
	  if( isset($newparms['name_type']) && $newparms['name_type'] == 'like' ) {
		$newparms['name'] = '::*'.$newparms['name'].'*';
		unset($newparms['name_type']);
	  }
	}
	if( isset($newparms['name_type']) ) unset($newparms['name_type']);
	if( isset($newparms['addr_type']) ) unset($newparms['addr_type']);
	if( empty($newparms['lat']) || empty($newparms['long']) ) {
	  unset($newparms['lat']);
	  unset($newparms['long']);
	  unset($newparms['radius']);
	  unset($newparms['units']);
	}
	else if( !isset($newparms['radius']) ) {
	  unset($newparms['lat']);
	  unset($newparms['long']);
	  unset($newparms['radius']);
	  unset($newparms['units']);
	}
	else if( isset($newparms['units']) ) {
	  $newparms['radius'] .= $newparms['units'];
	  unset($newparms['units']);
	}
	if( isset($newparms['address']) ) $newparms['address'] = '*'.$newparms['address'].'*';

	$fields = null;
	if( isset($newparms['field']) ) {
	  $fields = $newparms['field'];
	  unset($newparms['field']);
	}

	// create the filter object and query.
	$filter = new cd_company_filter($newparms);
	if( is_array($fields) && count($fields) ) {
	  foreach( $fields as $one ) {
		$expr = '';
		if( isset($one['expr']) ) $expr = $one['expr'];
		$filter->add_field_expr($one['fldname'],$one['fldval'],$expr);
	  }
	}

	$query = new cd_company_query($filter);
	$pagination = $query->get_pagination();
	$pagination->set_extraparams(array('cd_dosearch'=>1));
	$page = $pagination->get_current_page();

	// give everything to smarty.
	$smarty->assign('filter',$filter);
	$smarty->assign('items',$query->get_results($id,$returnid));
	$smarty->assign('itemcount',$query->get_result_count());
	$smarty->assign('pagination',$pagination);

	$smarty->assign('pagetext',$this->Lang('page'));
	$smarty->assign('oftext',$this->Lang('of'));
	$smarty->assign('pagecount',$pagination->get_page_count());
	$smarty->assign('curpage',$page);
	$smarty->assign('hierarchy_list',array_flip(cd_utils::get_hierarchy_list()));

	if( $pagination->get_page_count() > 1 ) {
	  if( $page == 1 ) {
		$smarty->assign('firstlink',$this->Lang('firstpage'));
		$smarty->assign('prevlink',$this->Lang('prevpage'));
	  }
	  else {
		$smarty->assign('firstlink',$pagination->get_page_link(1,$this->Lang('firstpage')));
		$smarty->assign('prevlink',$pagination->get_page_link($pagination->get_current_page()-1,$this->Lang('prevpage')));
	  }
	  if( $page == $pagination->get_page_count() ) {
		$smarty->assign('lastlink',$this->Lang('lastpage'));
		$smarty->assign('nextlink',$this->Lang('nextpage'));
	  }
	  else {
		$smarty->assign('lastlink',$pagination->get_page_link($pagination->get_page_count(),$this->Lang('lastpage')));
		$smarty->assign('nextlink',$pagination->get_page_link($pagination->get_current_page()+1,$this->Lang('nextpage')));
	  }
	}

	$thetemplate = 'summary_'.$this->GetPreference(COMPANYDIR_PREF_DFLTSUMMARY_TEMPLATE);
	if( isset($params['summarytemplate']) ) $thetemplate = 'summary_'.trim($params['summarytemplate']);
	echo $this->ProcessTemplateFromDatabase($thetemplate);
	return;
  }
  catch( Exception $e ) {
	$errors[] = $e->GetMessage();
  }
} // submit or do_search

#
# Give everything to smarty
#
if( count($errors) ) $smarty->assign('errors',$errors);
$tmp = cd_utils::get_hierarchy_list(-1,FALSE);
if( is_array($tmp) && count($tmp) ) {
  $tmp2 = array(-1=>$this->Lang('any'));
  foreach( $tmp as $k => $v ) $tmp2[$v] = $k; // backwards intentionally.
  $smarty->assign('hierarchies',$tmp2);
}

$tmp = $this->GetCategories();
if( is_array($tmp) ) {
  $categories = array();
  foreach( $tmp as $one ) {
	$categories[$one->id] = $one->name;
  }
  asort($categories);
  $smarty->assign('categories',$categories);
}

if( !isset($params['cd_origpage']) ) $params['cd_origpage'] = $returnid;
$smarty->assign('formstart',$this->CGCreateFormStart($id,'search',$resultpage,$params));
$smarty->assign('formend',$this->CreateFormEnd());
$smarty->assign('input_country',$this->CreateInputCountryDropdown($id,'cd_country'));

#
# Process the template
#
$thetemplate = $this->GetPreference(COMPANYDIR_SEARCHFORM_DFLTTEMPLATENAME);
if( isset($params['searchformtemplate']) ) $thetemplate = trim($params['searchformtemplate']);
echo $this->ProcessTemplateFromDatabase('newsearch_'.$thetemplate);

#
# EOF
#
?>