<?php  /* -*- Mode: PHP; tab-width: 4; c-basic-offset: 2 -*- */
#BEGIN_LICENSE
#-------------------------------------------------------------------------
# Module: CompanyDirectory (c) 2004-6 by Ted Kulp (ted@cmsmadesimple.org)
#  
#  Copyright 2006,2008,2009,2010 by Robert Campbell (calguy1000@cmsmadesimple.org)
#
#  An addon module for CMS Made Simple to allow management of and various ways to display
#  company information for use in directories etc.
# 
#-------------------------------------------------------------------------
# CMS - CMS Made Simple is (c) 2005-2010 by Ted Kulp (wishy@cmsmadesimple.org)
# This project's homepage is: http://www.cmsmadesimple.org
#
#-------------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# However, as a special exception to the GPL, this software is distributed
# as an addon module to CMS Made Simple.  You may not use this software
# in any Non GPL version of CMS Made simple, or in any version of CMS
# Made simple that does not indicate clearly and obviously in its admin 
# section that the site was built with CMS Made simple.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# Or read it online: http://www.gnu.org/licenses/licenses.html#GPL
#
#-------------------------------------------------------------------------
#END_LICENSE

class cd_ascii_importer
{
  private $_errors;
  private $_delimiter;
  private $_mod;
  private $_do_fielddefs;
  private $_do_fieldvals;
  private $_do_categorydefs;
  private $_do_categoryvals;
  private $_do_latlong_lookup;
  private $_create_url = 0;
  private $_check_duplicate_companies;
  private $_max_records;
  private $_do_hierdefs;
  private $_do_hiervals;
  private $_the_user;
  private $_categorycolumn;

  private $_num_companies;
  private $_num_hier;
  private $_num_categories;
  private $_num_fielddefs;
  private $_num_lines;
  private $_linenum = 0;

  private $_hier_cache;
  private $_fielddef_cache;
  private $_category_cache;
  private $_rules = array();

  public function __construct(&$mod,$delimiter = '|')
  {
    $this->_errors = array();

    $this->_do_fielddefs = 0;
    $this->_do_fieldvals = 0;
    $this->_the_user = -1;
    
    $this->_do_categorydefs = 0;
    $this->_do_categoryvals = 0;

    $this->_do_hierdefs = 0;
    $this->_do_hiervals = 0;

    $this->_do_latlong_lookup = 0;
    $this->_check_duplicate_companies = 'error';
    $this->_mod =& $mod;
    $this->_delimiter = $delimiter;
    $this->_max_records = 0;

    $this->_num_companies = 0;
    $this->_num_categories = 0;
    $this->_num_fielddefs = 0;
    $this->_num_hier = 0;
    $this->_num_lines = 0;
  }

  public function get_results()
  {
    return array('num_companies'=>$this->_num_companies,
		 'num_categories'=>$this->_num_categories,
		 'num_fielddefs'=>$this->_num_fielddefs,
		 'num_hier'=>$this->_num_hier,
		 'num_lines'=>$this->_num_lines);
  }

  public function set_user($uid)
  {
    $this->_the_user = $uid;
  }

  public function set_delimiter($char)
  {
    $this->_delimiter = $char;
  }

  public function set_max_companies($num)
  {
    $this->_max_records = $num;
  }

  public function do_fielddefs($flag = true)
  {
    $this->_do_fielddefs = $flag;
  }

  public function do_fieldvals($flag = true)
  {
    $this->_do_fieldvals = $flag;
  }

  public function do_categorydefs($flag = true)
  {
    $this->_do_categorydefs = $flag;
  }

  public function do_categoryvals($flag = true)
  {
    $this->_do_categoryvals = $flag;
  }

  public function do_latlong_lookup($flag = true )
  {
    $this->_do_latlong_lookup = $flag;
  }

  public function create_url($flag = true)
  {
	$this->_create_url = $flag;
  }

  public function check_duplicate_companies($flag = true)
  {
	$tmp = strtolower($flag);
	switch( $tmp ) {
	case 'error':
	  // generate an error on duplicate companies
	case 'ignore':
	  // just ignore the whole record, silently
	case 'update':
	  // update the company.
	  $this->_check_duplicate_companies = $tmp;
	  break;
	}
  }

  public function do_hierarchyvals($flag = true)
  {
    $this->_do_hiervals = $flag;
  }

  public function do_hierarchydefs($flag = true)
  {
    $this->_do_hierdefs = $flag;
  }

  public function set_categorycolumn($str = null)
  {
	$this->_categorycolumn = $str;
  }

  protected function set_error($msg)
  {
    $str = $this->_mod->Lang('error_prefix_msg',$this->_linenum,$msg);
    $this->_errors[] = $str;
  }

  public function get_errors()
  {
    return $this->_errors;
  }

  public function clear_errors()
  {
    $this->_errors = array();
  }

  protected function get_fielddef_id($name,$insert = false)
  {
    $db = cmsms()->GetDb();

    if( !is_array($this->_fielddef_cache) ) {
	  $query = 'SELECT id,name FROM '.cms_db_prefix().'module_compdir_fielddefs ORDER BY item_order';
	  $tmp = $db->GetArray($query);
	  if( is_array($tmp) ) {
		$this->_fielddef_cache = cge_array::to_hash($tmp,'name');
	  }
	}

    if( is_array($this->_fielddef_cache) && isset($this->_fielddef_cache[$name]) ) {
	  return $this->_fielddef_cache[$name]['id'];
	}

	if( $insert ) {
	  // insert that record
	  $item_order = $db->GetOne('SELECT max(item_order) + 1 FROM ' . cms_db_prefix() . 'module_compdir_fielddefs');
	  $query = "INSERT INTO ".cms_db_prefix()."module_compdir_fielddefs
               (name, type, max_length, admin_only, public, item_order, create_date, modified_date)
              VALUES (?,?,?,?,?,?,NOW(),NOW())";
	  $dbr = $db->Execute($query,array($name,'textbox',255,0,1,$item_order));
	  if( !$dbr ) {
		debug_display($db->sql);
		debug_display($db->ErrorMsg());
		die();
	  }
		
	  $fd_id = $db->Insert_ID();
	  $this->_fielddef_cache[$name]['id'] = $fd_id;
	  $this->_num_fielddefs++;
	  return $fd_id;
	}

    return FALSE;
  }

  
  protected function get_hier_id($long_name,$refresh_cache = false)
  {
    $db = cmsms()->GetDb();

    if( !is_array($this->_hier_cache) || $refresh_cache == true ) {
      $query = 'SELECT * FROM '.cms_db_prefix().'module_compdir_hier ORDER BY hierarchy';
      $tmp = $db->GetArray($query);
      if( is_array($tmp) ) {
		$tmp2 = cge_array::to_hash($tmp,'long_name');
		$this->_hier_cache = $tmp2;
	  }
      else {
		return FALSE;
	  }
    }

    if( !isset($this->_hier_cache[$long_name]) ) return FALSE;
    return $this->_hier_cache[$long_name]['id'];
  }


  protected function insert_hier($long_name)
  {
    if( empty($long_name) ) return;

    $db = cmsms()->GetDb();
    $query = 'INSERT INTO '.cms_db_prefix().'module_compdir_hier
              (name,parent_id,long_name) VALUES (?,?,?)';
    $pos = cge_array::smart_explode($long_name,' | ');
    $tmp = array();
    $parent = -1;
    for( $i = 0; $i < count($pos); $i++ ) {
	  $tmp[] = $pos[$i];
	  $tmp2 = implode(' | ',$tmp);
	  $name = trim($pos[$i]);
	  $this_id = $this->get_hier_id($tmp2,true);
	  if( $this_id === FALSE && $name != '') {
	    // gotta insert it.
	    $dbr = $db->Execute($query,array($name,$parent,$tmp2));
	    $this->_num_hier++;
	  }
	  $parent = $this->get_hier_id($tmp2,true);
	}
  }
  

  protected function get_category_id($name,$insert = false)
  {
    $db = cmsms()->GetDb();

    if( !is_array($this->_category_cache) ) {
	  $query = 'SELECT id,name FROM '.cms_db_prefix().'module_compdir_categories ORDER BY iorder';
	  $tmp = $db->GetArray($query);
	  if( is_array($tmp) ) {
		$this->_category_cache = cge_array::to_hash($tmp,'name');
	  }
	}

    if( is_array($this->_category_cache) && isset($this->_category_cache[$name]) ) {
	  return $this->_category_cache[$name]['id'];
	}

	if( $insert ) {
	  $query = 'INSERT INTO '.cms_db_prefix().'module_compdir_categories 
                (name,create_date,modified_date)
                VALUES (?,NOW(),NOW())';
	  $dbr = $db->Execute($query,array($name));
	  if( !$dbr ) {
		debug_display($db->sql);
		debug_display($db->ErrorMsg());
		die();
	  }
	  $new_id = $db->Insert_ID();

	  $this->_category_cache[$name] = array('id'=>$new_id,'name'=>$name);
	  $this->_num_categories++;
	  return $new_id;
	}

    return FALSE;
  }


  private function unprocess_data($data)
  {
    // strip quotes
    $data = trim($data,'"');
    
    // newlines
    $data = str_replace("^^","\n",$data);
    $data = str_replace("-%%-",$this->_delimiter,$data);
    
    return $data;
  }


  private function process_fielddef($line,$rule)
  {
    $db = cmsms()->GetDb();

    // break the line into fields again
    $fields = cge_array::smart_explode($line,$this->_delimiter);

    // build a new record
    $record = array();
    $record['dropdown_data'] = '';
    for( $fld = 1; $fld < count($rule['flds']); $fld++ ) {
	  if( !isset($fields[$fld]) ) continue;
	  $fname = $rule['flds'][$fld];
	  $record[$fname] = $this->unprocess_data($fields[$fld]);
	}

    // check for duplicate
    $tmp = $this->get_fielddef_id($record['name']);
    if( $tmp ) {
	  $this->set_error($this->_mod->Lang('error_duplicate_fielddef'));
	  return FALSE;
	}

    // insert that record
    $now = $db->DbTimeStamp(time());
    $item_order = $db->GetOne('SELECT max(item_order) + 1 FROM ' . cms_db_prefix() . 'module_compdir_fielddefs');
    $query = "INSERT INTO ".cms_db_prefix()."module_compdir_fielddefs
              (name, type, max_length, admin_only, public, dropdown_data, item_order, 
               create_date, modified_date)
              VALUES (?,?,?,?,?,?,?,$now,$now)";
    $dbr = $db->Execute($query,
						array($record['name'],$record['type'],$record['max_length'],
							  $record['admin_only'],$record['public'],
							  $record['dropdown_data'],$item_order));

    // trap the error.
    if( !$dbr ) {
	  $this->set_error($this->_mod->Lang('error_insert_fielddef'));
	  return FALSE;
	}

	$fd_id = $db->Insert_ID();
	if( !is_array( $this->_fielddef_cache ) ) {
	  $this->_fielddef_cache = array();
	}
	$this->_fielddef_cache[$record['name']] = array('id'=>$fd_id);
	$this->_num_fielddefs++;
    return TRUE;
  }

  
  private function process_hierarchy($line,$rule)
  {
    $db = cmsms()->GetDb();
    $fields = explode($this->_delimiter,$line);

    $record = array();
    for( $fld = 1; $fld < count($rule['flds']); $fld++ ) {
	  if( !isset($fields[$fld]) ) continue;
	  $fname = $rule['flds'][$fld];
	  $record[$fname] = $this->unprocess_data($fields[$fld]);
	}

    // check for duplicate
    $tmp = $this->get_hier_id($record['long_name']);
    if( $tmp ) {
	  $this->set_error($this->_mod->Lang('error_duplicate_hierarchy',$record['long_name']));
	  return FALSE;
	}

    // insert the record
    $tmp = $this->insert_hier($record['long_name']);
    if( $tmp ) {
	  $this->set_error($this->_mod->Lang('error_insert_hierarchy',$record['long_name']));
	  return FALSE;
	}

    return TRUE;
  }


  private function process_category($line,$rule)
  {
    $db = cmsms()->GetDb();

    // break the line into fields again
    $fields = explode($this->_delimiter,$line);

    // build a new record
    $record = array();
    for( $fld = 1; $fld < count($rule['flds']); $fld++ ) {
	  if( !isset($fields[$fld]) ) continue;
	  $fname = $rule['flds'][$fld];
	  $record[$fname] = $this->unprocess_data($fields[$fld]);
	}

    // check for duplicate
    $tmp = $this->get_category_id($record['name']);
    if( $tmp ) {
	  $this->set_error($this->_mod->Lang('error_duplicate_category'));
	  return FALSE;
	}
  
    // insert that record
    $now = $db->DbTimeStamp(time());
    $query = "INSERT INTO ".cms_db_prefix()."module_compdir_categories
              (name,create_date,modified_date)
              VALUES (?,$now,$now)";
    $dbr = $db->Execute($query,array($record['name']));
  
    // trap the error.
    if( !$dbr ) {
	  $this->set_error($this->_mod->Lang('error_insert_category'));
	  return FALSE;
	}

	$cat_id = $db->Insert_ID();
	if( is_array($this->_category_cache) ) {
	  $this->_category_cache = array();
	}
	$this->_category_cache[$record['name']] = array('id'=>$cat_id);
	$this->_num_categories++;
    return TRUE;
  }

  
  private function process_company($fields,$rule)
  {
    $db = cmsms()->GetDb();
    $userid = $this->_the_user;

    // build a new record
    $record = array();
    $record['contact_email'] = '';
	$record['url'] = '';
    $record['details'] = '';
    $record['status'] = 'published';
    $record['latitude'] = '';
    $record['longitude'] = '';
    $record['hier_id'] = null;
	$record['telephone'] = '';
	$record['website'] = '';
	$record['fax'] = '';

    $fieldvals = array();
    $categories = array();
    for( $fld = 1; $fld < count($rule['flds']); $fld++ ) {
	  if( !isset($fields[$fld]) ) continue;
	  $fname = $rule['flds'][$fld];
		
	  // primary column handling.
	  if( startswith(strtoupper($fname),'FIELD:') ) {
		$fieldvals[substr($fname,6)] = $fields[$fld];
	  }
	  else if( startswith(strtoupper($fname),'CAT:') ) {
		$categories[substr($fname,4)] = $fields[$fld];
	  }
	  else if( $fname == 'hierarchy' && $this->_do_hiervals) {
		$tmp = $this->unprocess_data($fields[$fld]);
		$hier_id = $this->get_hier_id($tmp);
		if( $hier_id === FALSE && $this->_do_hiervals ) {
		  $this->insert_hier($tmp);
		}
		$hier_id = $this->get_hier_id($tmp);
		if( $hier_id !== FALSE )
		  $record['hier_id'] = $hier_id;
	  }
	  else {
		$record[$fname] = $this->unprocess_data($fields[$fld]);
	  }

	  // additional handling of this column
	  if( $this->_categorycolumn != '' && $fname == $this->_categorycolumn ) {
		$cats = cge_array::smart_explode($fields[$fld],',');
		foreach( $cats as $one ) {
	          $one = trim($one);
		  $categories[$one] = 1;
		}
	  }

	} //for

  
    // data validation
    if( !isset($record['company_name']) || empty($record['company_name']) ) {
	  $this->set_error($this->_mod->Lang('error_import_badvalue','company_name'));
	  return FALSE;
	}

    // do optional lat/long lookup
    if( $this->_do_latlong_lookup ) {
	  if( empty($record['latitude']) || empty($record['longitude']) && 
		  !empty($record['address']) ) {
		$data = cd_utils::geolocate($record['address']);
		if( $data && $data['lat'] != 0 && $data['lon'] != 0 ) {
		  $record['latitude'] = $data['lat'];
		  $record['longitude'] = $data['lon'];
		}
	  }
	}

	// handle url
	if( $record['url'] == '' && $this->_create_url ) {
	  $record['url'] = cd_utils::generate_url($record['company_name']);
	}
	if( $record['url'] != '' ) {
	  while( !cd_utils::validate_url($record['url'],TRUE) ) {
		$record['url'] = cd_utils::generate_url($record['company_name']);
	  }
	}
	
    // check for duplicate?
	$query = 'SELECT id FROM '.cms_db_prefix().'module_compdir_companies WHERE company_name = ?';
	$eid = $db->GetOne($query,array($record['company_name']));
	if( $eid ) {
	  switch( $this->_check_duplicate_companies ) {
	  case 'error':
		$this->set_error($this->_mod->Lang('error_duplicate_company',$record['company_name']));
		return FALSE;
	  case 'ignore':
		return FALSE;
	  case 'update':
		// overwriting... keep going.
		break;
	  }
	}
  
    // insert that record
    $now = $db->DbTimeStamp(time());
	$dbr = TRUE;
	if( $eid ) {
	  $query = 'UPDATE '.cms_db_prefix()."module_compdir_companies 
                SET url = ?, address = ?, telephone = ?, fax = ?, contact_email = ?, 
                    website = ?, details = ?, status = ?, latitude = ?,
                    longitude = ?, owner_id = ?, hier_id = ?, modified_date = ? WHERE id = ?";
	  $dbr = $db->Execute($query,
						  array($record['url'],$record['address'],$record['telephone'],
								$record['fax'],$record['contact_email'],$record['website'],
								$record['details'],$record['status'],
								$record['latitude'],$record['longitude'],
								$userid,$record['hier_id'],$now,$eid));

	  if( !$dbr ) {
		$this->set_error($this->_mod->Lang('error_update_company'));
		return FALSE;
	  }

	  $query = 'DELETE FROM '.cms_db_prefix().'module_compdir_fieldvals WHERE company_id = ?';
	  $dbr = $db->Execute($query,array($eid));

	  $query = 'DELETE FROM '.cms_db_prefix().'module_compdir_company_categories WHERE company_id = ?';
	  $dbr = $db->Execute($query,array($eid));

	  $company_id = $eid;
	}
	else {
	  $query =  "INSERT INTO ".cms_db_prefix()."module_compdir_companies 
                 (company_name,url,address,telephone,fax,contact_email,website,details,status,
                  latitude,longitude,owner_id, hier_id,create_date, modified_date)
                 VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,$now,$now)";
	  $dbr = $db->Execute($query,array($record['company_name'],$record['url'],$record['address'],
									   $record['telephone'],$record['fax'],
									   $record['contact_email'],$record['website'],
									   $record['details'],$record['status'],
									   $record['latitude'],$record['longitude'],
									   $userid,$record['hier_id']));
	  if( !$dbr ) {
		$this->set_error($this->_mod->Lang('error_insert_company'));
		return FALSE;
	  }
	  $company_id = $db->Insert_ID();
	}

	// insert fieldvals
	if( count($fieldvals) && $this->_do_fieldvals ) {
	  foreach( $fieldvals as $name => $value ) {
		if( empty($value) ) continue;
		$fielddef_id = $this->get_fielddef_id($name,$this->_do_fielddefs);
		if( $fielddef_id === FALSE ) continue;
			
		$query = 'INSERT INTO '.cms_db_prefix()."module_compdir_fieldvals 
                  (company_id, fielddef_id, value, create_date, modified_date)
                  VALUES (?,?,?,$now,$now)";
		$dbr = $db->Execute($query,array($company_id,$fielddef_id,$value));
	  }
	}
  
	// insert categories
	if( count($categories) && $this->_do_categoryvals ) {
	  foreach($categories as $name => $value ) {
		if( $value == 0 ) continue;
		$category_id = $this->get_category_id($name,$this->_do_categorydefs);
		if( $category_id === FALSE ) continue;
			
		$query = 'INSERT INTO '.cms_db_prefix()."module_compdir_company_categories
                  (company_id,category_id,create_date,modified_date)
                  VALUES (?,?,$now,$now)";
		$db->Execute($query,array($company_id,$category_id));
	  }
	}
	
    return TRUE;
  }

  private function _is_header($line,$add = FALSE)
  {
	$fields = cge_array::smart_explode($line,$this->_delimiter);
	if( !is_array($fields) || count($fields) == 0 ) return FALSE;
	foreach( $fields as &$fld ) $fld = trim($fld);
	$key = $fields[0];

	if(startswith($fields[0],'#COMPANY')) {
	  // company rule
	  if( isset($this->_rules['company']) && $add == TRUE) {
		// this is bad.
		$this->set_error($this->_mod->Lang('error_import_duplicaterule','COMPANY',$this->_linenum));
		return TRUE;
	  }

	  if( $this->_categorycolumn != '' && !in_array($this->_categorycolumn,$fields) && $add == TRUE ) {
		$this->set_error($this->_mod->Lang('error_categorycolumn_notfound',$this->_categorycolumn));
		return TRUE;
	  }

	  if( $add ) {
		$tmp = explode('=',$fields[0],2);
		$identifier = $tmp[1];

		$this->_rules['company'] = array();
		$this->_rules['company']['rule'] = $line;
		$this->_rules['company']['identifier'] = $identifier;
		$this->_rules['company']['flds'] = $fields;
	  }
	  return TRUE;
	}

	if( startswith($fields[0],'#FIELDDEF')) {
	  // fielddef rule
	  if( isset($this->_rules['fielddef']) && $add ) {
		// this is bad.
		$this->set_error($this->_mod->Lang('error_import_duplicaterule','FIELDDEF',$this->_linenum));
		return TRUE;
	  }

	  if( $add ) {
		$tmp = explode('=',$fields[0],2);
		$identifier = $tmp[1];

		$this->_rules['fielddef'] = array();
		$this->_rules['fielddef']['rule'] = $line;
		$this->_rules['fielddef']['identifier'] = $identifier;
		$this->_rules['fielddef']['flds'] = $fields;
	  }
	  return TRUE;
	}

	if( startswith($fields[0],'#CATEGORY')) {
	  // category rule
	  if( isset($this->_rules['category']) && $add ) {
		// this is bad.
		$this->set_error($this->_mod->Lang('error_import_duplicaterule','CATEGORY',$this->_linenum));
		return TRUE;
	  }

	  if( $add ) {
		$tmp = explode('=',$fields[0],2);
		$identifier = $tmp[1];

		$this->_rules['category'] = array();
		$this->_rules['category']['rule'] = $line;
		$this->_rules['category']['identifier'] = $identifier;
		$this->_rules['category']['flds'] = $fields;
	  }
	  return TRUE;
	}
	
	if( startswith($fields[0],'#HIERARCHY')) {
	  // hierarchy rule
	  if( isset($this->_rules['hierarchy']) && $add ) {
		// this is bad.
		$this->set_error($this->_mod->Lang('error_import_duplicaterule','HIERARCHY',$this->_linenum));
		return TRUE;
	  }

	  if( $add ) {
		$tmp = explode('=',$fields[0],2);
		$identifier = $tmp[1];

		$this->_rules['hierarchy'] = array();
		$this->_rules['hierarchy']['rule'] = $line;
		$this->_rules['hierarchy']['identifier'] = $identifier;
		$this->_rules['hierarchy']['flds'] = $fields;
	  }
	  return TRUE;
	}

	if( startswith($line,'#') ) {
	  // don't know this one.
	  $this->set_error($this->_mod->Lang('error_import_unknownrule',$this->_linenum));
	  return TRUE;
	}

	return FALSE;
  }

  private function _is_record($line)
  {
	if( $this->_is_header($line,TRUE) ) return TRUE;

	$fields = cge_array::smart_explode($line,$this->_delimiter);
	if( !is_array($fields) || count($fields) == 0 ) return FALSE;
	foreach( $fields as &$fld ) $fld = trim($fld);
	$key = $fields[0];

	foreach( $this->_rules as $name => $data ) {
	  if( $key == $this->_rules[$name]['identifier'] ) return TRUE;
	}

	// doesn't match a rule (yet)
	return FALSE;
  }

  private function _getline($fh)
  {
	$in_rec = FALSE;
	$out = '';
	while( !feof($fh) ) {
	  $pos = ftell($fh);
	  $line = fgets($fh);
	  $this->_linenum++;
	  $line = trim($line);
	  if( $line == '' ) continue;

	  if( $this->_is_record($line) ) {
		// start of a new record
		if( $in_rec ) {
		  // rewind a bit.
		  fseek($fh,$pos);
		  $this->_linenum--;
		  $in_rcc = FALSE;
		  // done.
		  break;
		}
		else {
		  $in_rec = TRUE;
		  $out .= $line;
		}
	  }
	  else {
		$out .= $line;
	  }
	}
	return $out;
  }

  public function import_file($filename)
  {
    $this->_linenum = 0;
    $fh = fopen($filename,'r');
    if( !$fh ) {
	  $this->set_error($this->_mod->Lang('error_badupload'));
	  return FALSE;
	}
    while( !feof($fh) ) {
	  $line = $this->_getline($fh);
		
	  // strip off extra whitespace
	  $line = trim($line);
	  
	  // skip empty lines
	  if ($line == '') {
		continue;
	  }

	  //
	  // handle definition lines
	  //
	  $fields = cge_array::smart_explode($line,$this->_delimiter);

	  //
	  // do real importing
	  //
	  // make sure it's not a header line.
	  if( $this->_is_header($line) ) continue;

	  $rowtype = '';
	  foreach( $this->_rules as $name => $data ) {
	    if( $fields[0] == $this->_rules[$name]['identifier'] ) {
		  $rowtype = $name;
		  break;
		}
	  }
	  if( empty($rowtype) ) {
		debug_display($line,'error at '.$this->_linenum); die();
	    $this->set_error($this->_mod->Lang('error_import_unknownrow',$this->_linenum,$fields[0]));
	    continue;
	  }

	  $ret = '';
	  switch( $rowtype ) {
	  case 'company':
	    if( ($this->_max_records > 0 && $this->_num_companies < $this->_max_records)
			|| $this->_max_records == 0 ) {
		  $ret = $this->process_company($fields,$this->_rules['company']);
		  if( $ret === TRUE ) $this->_num_companies++;
		}
	    continue;

	  case 'category':
	    if( $this->_do_categorydefs ) {
		  $ret = $this->process_category($line,$this->_rules['category']);
		  if( $ret === TRUE ) $this->_num_categories++;
		}
	    continue;
	    break;

	  case 'fielddef':
	    if( $this->_do_fielddefs ) {
		  $ret = $this->process_fielddef($line,$this->_rules['fielddef']);
		  if( $ret === TRUE ) $this->_num_fielddefs++;
		}
	    continue;
	    break;

	  case 'hierarchy':
	    if( $this->_do_hierdefs ) {
		  $ret = $this->process_hierarchy($line,$this->_rules['hierarchy']);
		  if( $ret === TRUE ) $this->_num_hier++;
		}
	    continue;
	    break;

	  default:
		if( $rowtype != '' ) {
		  $this->set_error($this->_mod->Lang('error_import_unknownrow',$this->_linenum,$fields[0]));
		}
	    continue;
	    break;
	  }
	}
    $this->_num_lines = $this->_linenum;
    return TRUE;
  }

}

#
# EOF
#
?>
